"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handlePort = exports.handleSchedule = exports.handleSelector = exports.handleOperation = exports.handlerOperations = exports.useRunner = exports.makePromiseResult = exports.api = void 0;
const debug_1 = __importDefault(require("debug"));
const client_1 = require("@deep-foundation/hasura/client");
const api_1 = require("@deep-foundation/hasura/api");
const apollo_boost_1 = require("apollo-boost");
const promise_1 = require("../promise");
const client_2 = require("../client");
const global_ids_1 = require("../global-ids");
const container_controller_1 = require("../container-controller");
const SCHEMA = 'public';
const debug = (0, debug_1.default)('deeplinks:eh:links');
const log = debug.extend('log');
const error = debug.extend('error');
const namespaces = debug_1.default.disable();
debug_1.default.enable(`${namespaces ? `${namespaces},` : ``}${error.namespace}`);
const DOCKER_DEEPLINKS_URL = process.env.DOCKER_DEEPLINKS_URL || 'http://host.docker.internal:3006';
const DOCKER = process.env.DOCKER || '0';
exports.api = new api_1.HasuraApi({
    path: process.env.DEEPLINKS_HASURA_PATH,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const client = (0, client_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_2.DeepClient({
    apolloClient: client,
});
function makePromiseResult(promise, resolvedTypeId, promiseResultTypeId, result, promiseReasonTypeId, handleInsertId) {
    return {
        from: {
            data: {
                from_id: promise.id,
                type_id: resolvedTypeId,
                to: {
                    data: {
                        type_id: promiseResultTypeId,
                        object: { data: { value: result } },
                    }
                }
            }
        },
        type_id: promiseReasonTypeId,
        to_id: handleInsertId
    };
}
exports.makePromiseResult = makePromiseResult;
;
const containerController = new container_controller_1.ContainerController({
    gql_docker_domain: +DOCKER ? 'links' : 'graphql-engine',
    gql_port_path: +DOCKER ? '3006/gql' : '8080/v1/graphql',
    network: 'deep_network',
    handlersHash: {}
});
const useRunner = ({ code, handler, oldLink, newLink, moment, promiseId, }) => __awaiter(void 0, void 0, void 0, function* () {
    const useRunnerDebug = (0, debug_1.default)('deeplinks:eh:links:useRunner');
    useRunnerDebug("handler4: ");
    const jwt = (yield deep.jwt({ linkId: yield deep.id('@deep-foundation/core', 'system', 'admin') })).token;
    useRunnerDebug('jwt', jwt);
    const container = yield containerController.newContainer({ publish: +DOCKER ? false : true, forceRestart: true, handler, code, jwt, data: { oldLink, newLink, moment } });
    useRunnerDebug('newContainerResult', container);
    const initResult = yield containerController.initHandler(container);
    useRunnerDebug('initResult', initResult);
    const callResult = yield containerController.callHandler({ code, container, jwt, data: { oldLink, newLink, moment, promiseId } });
    useRunnerDebug('callResult', callResult);
    return callResult;
});
exports.useRunner = useRunner;
exports.handlerOperations = {
    Insert: 'HandleInsert',
    Update: 'HandleUpdate',
    Delete: 'HandleDelete',
};
function handleOperation(operation, oldLink, newLink) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    return __awaiter(this, void 0, void 0, function* () {
        const handleOperationDebug = (0, debug_1.default)('deeplinks:eh:links:handleOperation');
        const current = newLink !== null && newLink !== void 0 ? newLink : oldLink;
        const currentLinkId = current.id;
        const currentTypeId = current.type_id;
        const handlerTypeId = yield deep.id('@deep-foundation/core', 'Handler');
        const handleOperationTypeId = yield deep.id('@deep-foundation/core', exports.handlerOperations[operation]);
        const dockerSupportsJsType = yield deep.id('@deep-foundation/core', 'dockerSupportsJs');
        const queryString = `query SELECT_CODE($typeId: bigint) { links(where: {
          type_id: { _eq: ${yield deep.id('@deep-foundation/core', 'SyncTextFile')} },
          in: {
            from_id: { _eq: ${dockerSupportsJsType} },
            type_id: { _eq: ${handlerTypeId} },
            in: {
              from_id: { _eq: $typeId },
              type_id: { _eq: ${handleOperationTypeId} },
            }
          }
        }) {
          id
          value
          in(where: { type_id: { _eq: ${handlerTypeId} } }) {
            id
            in(where: { type_id: { _eq: ${handleOperationTypeId} } }) {
              id
            }
            support: from {
              id
              isolation: from {
                id
                value
              }
            }
          }
        } }`;
        const query = (0, apollo_boost_1.gql) `${queryString}`;
        const variables = {
            typeId: currentTypeId
        };
        const handlersResult = yield client.query({ query, variables });
        const promises = [];
        const handleInsertsIds = [];
        const handlersWithCode = (_a = handlersResult === null || handlersResult === void 0 ? void 0 : handlersResult.data) === null || _a === void 0 ? void 0 : _a.links;
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const rejectedTypeId = yield deep.id('@deep-foundation/core', 'Rejected');
        const promiseResultTypeId = yield deep.id('@deep-foundation/core', 'PromiseResult');
        const promiseReasonTypeId = yield deep.id('@deep-foundation/core', 'PromiseReason');
        const promise = yield (0, promise_1.findPromiseLink)({
            id: currentLinkId, client: deep.apolloClient,
            Then: yield deep.id('@deep-foundation/core', 'Then'),
            Promise: yield deep.id('@deep-foundation/core', 'Promise'),
            Resolved: resolvedTypeId,
            Rejected: rejectedTypeId,
            Results: false,
        });
        if ((handlersWithCode === null || handlersWithCode === void 0 ? void 0 : handlersWithCode.length) > 0) {
            handleOperationDebug("handlersWithCode: ", JSON.stringify(handlersWithCode, null, 2));
            handleOperationDebug("handlersWithCode?.length: ", handlersWithCode === null || handlersWithCode === void 0 ? void 0 : handlersWithCode.length);
            for (const handlerWithCode of handlersWithCode) {
                const code = (_b = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.value) === null || _b === void 0 ? void 0 : _b.value;
                const isolationValue = (_g = (_f = (_e = (_d = (_c = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.in) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.support) === null || _e === void 0 ? void 0 : _e.isolation) === null || _f === void 0 ? void 0 : _f.value) === null || _g === void 0 ? void 0 : _g.value;
                const handleInsertId = (_k = (_j = (_h = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.in) === null || _h === void 0 ? void 0 : _h[0]) === null || _j === void 0 ? void 0 : _j.in) === null || _k === void 0 ? void 0 : _k[0].id;
                if (code) {
                    try {
                        promises.push(() => __awaiter(this, void 0, void 0, function* () { return (0, exports.useRunner)({ code, handler: isolationValue, oldLink, newLink, promiseId: promise.id }); }));
                        handleInsertsIds.push(handleInsertId);
                    }
                    catch (error) {
                        handleOperationDebug('error', error);
                    }
                }
            }
            handleOperationDebug('promise: ', promise);
            if (promise) {
                handleOperationDebug("promises.length: ", promises.length);
                yield Promise.allSettled(promises.map((p) => p()))
                    .then((values) => __awaiter(this, void 0, void 0, function* () {
                    var _l;
                    handleOperationDebug("values: ", values);
                    const promiseResults = [];
                    for (let i = 0; i < values.length; i++) {
                        const value = values[i];
                        const handleInsertId = handleInsertsIds[i];
                        if (value.status == 'fulfilled') {
                            const result = value.value;
                            handleOperationDebug("result: ", result);
                            const promiseResult = makePromiseResult(promise, resolvedTypeId, promiseResultTypeId, result, promiseReasonTypeId, handleInsertId);
                            promiseResults.push(promiseResult);
                        }
                        if (value.status == 'rejected') {
                            const error = value.reason;
                            handleOperationDebug("error: ", error);
                            const promiseResult = makePromiseResult(promise, rejectedTypeId, promiseResultTypeId, error, promiseReasonTypeId, handleInsertId);
                            promiseResults.push(promiseResult);
                        }
                    }
                    try {
                        yield deep.insert(promiseResults, { name: 'IMPORT_PROMISES_RESULTS' });
                        handleOperationDebug("inserted promiseResults: ", JSON.stringify(promiseResults, null, 2));
                    }
                    catch (e) {
                        handleOperationDebug('promiseResults insert error: ', (_l = e === null || e === void 0 ? void 0 : e.message) !== null && _l !== void 0 ? _l : e);
                    }
                }));
            }
        }
    });
}
exports.handleOperation = handleOperation;
function handleSelector(oldLink, newLink) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    return __awaiter(this, void 0, void 0, function* () {
        const handleSelectorDebug = debug.extend('handleSelector').extend('log');
        const current = newLink !== null && newLink !== void 0 ? newLink : oldLink;
        const currentLinkId = current.id;
        const currentTypeId = current.type_id;
        handleSelectorDebug('currentLinkId', currentLinkId);
        handleSelectorDebug('currentTypeId', currentTypeId);
        const handlerTypeId = yield deep.id('@deep-foundation/core', 'Handler');
        const selectorTypeId = yield deep.id('@deep-foundation/core', 'Selector');
        const handleOperationTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
        const dockerSupportsJsType = yield deep.id('@deep-foundation/core', 'dockerSupportsJs');
        handleSelectorDebug('handlerTypeId', handlerTypeId);
        handleSelectorDebug('handleOperationTypeId', handleOperationTypeId);
        const queryString = `query SELECT_CODE($typeId: bigint) { links(where: {
          type_id: { _eq: ${yield deep.id('@deep-foundation/core', 'SyncTextFile')} },
          in: {
            from_id: { _eq: ${dockerSupportsJsType} },
            type_id: { _eq: ${handlerTypeId} },
            in: {
              from: {
                type_id: { _eq: ${selectorTypeId} },
                selected: {
                  item_id: { _eq: ${currentLinkId} },
                },
              },
              type_id: { _eq: ${handleOperationTypeId} },
            }
          }
        }) {
          id
          value
          in(where: { type_id: { _eq: ${handlerTypeId} } }) {
            id
            in(where: { type_id: { _eq: ${handleOperationTypeId} } }) {
              id
            }
            support: from {
              id
              isolation: from {
                id
                value
              }
            }
          }
        } }`;
        const query = (0, apollo_boost_1.gql) `${queryString}`;
        const variables = {
            typeId: currentTypeId
        };
        const handlersResult = yield client.query({ query, variables });
        const promises = [];
        const handleInsertsIds = [];
        const handlersWithCode = (_a = handlersResult === null || handlersResult === void 0 ? void 0 : handlersResult.data) === null || _a === void 0 ? void 0 : _a.links;
        handleSelectorDebug('handlersWithCode.length', handlersWithCode === null || handlersWithCode === void 0 ? void 0 : handlersWithCode.length);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const rejectedTypeId = yield deep.id('@deep-foundation/core', 'Rejected');
        const promiseResultTypeId = yield deep.id('@deep-foundation/core', 'PromiseResult');
        const promiseReasonTypeId = yield deep.id('@deep-foundation/core', 'PromiseReason');
        const promise = yield (0, promise_1.findPromiseLink)({
            id: currentLinkId, client: deep.apolloClient,
            Then: yield deep.id('@deep-foundation/core', 'Then'),
            Promise: yield deep.id('@deep-foundation/core', 'Promise'),
            Resolved: resolvedTypeId,
            Rejected: rejectedTypeId,
            Results: false,
        });
        if ((handlersWithCode === null || handlersWithCode === void 0 ? void 0 : handlersWithCode.length) > 0) {
            handleSelectorDebug("handlersWithCode: ", JSON.stringify(handlersWithCode, null, 2));
            handleSelectorDebug("handlersWithCode?.length: ", handlersWithCode === null || handlersWithCode === void 0 ? void 0 : handlersWithCode.length);
            for (const handlerWithCode of handlersWithCode) {
                const code = (_b = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.value) === null || _b === void 0 ? void 0 : _b.value;
                const isolationValue = (_g = (_f = (_e = (_d = (_c = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.in) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.support) === null || _e === void 0 ? void 0 : _e.isolation) === null || _f === void 0 ? void 0 : _f.value) === null || _g === void 0 ? void 0 : _g.value;
                const handleInsertId = (_k = (_j = (_h = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.in) === null || _h === void 0 ? void 0 : _h[0]) === null || _j === void 0 ? void 0 : _j.in) === null || _k === void 0 ? void 0 : _k[0].id;
                if (code) {
                    try {
                        promises.push(() => __awaiter(this, void 0, void 0, function* () { return (0, exports.useRunner)({ code, handler: isolationValue, oldLink, newLink, promiseId: promise.id }); }));
                        handleInsertsIds.push(handleInsertId);
                    }
                    catch (error) {
                        handleSelectorDebug('error', error);
                    }
                }
            }
            handleSelectorDebug('promise: ', promise);
            if (promise) {
                handleSelectorDebug("promises.length: ", promises.length);
                yield Promise.allSettled(promises.map((p) => p()))
                    .then((values) => __awaiter(this, void 0, void 0, function* () {
                    var _l;
                    handleSelectorDebug("values: ", values);
                    const promiseResults = [];
                    for (let i = 0; i < values.length; i++) {
                        const value = values[i];
                        const handleInsertId = handleInsertsIds[i];
                        if (value.status == 'fulfilled') {
                            const result = value.value;
                            handleSelectorDebug("result: ", result);
                            const promiseResult = makePromiseResult(promise, resolvedTypeId, promiseResultTypeId, result, promiseReasonTypeId, handleInsertId);
                            promiseResults.push(promiseResult);
                        }
                        if (value.status == 'rejected') {
                            const error = value.reason;
                            handleSelectorDebug("error: ", error);
                            const promiseResult = makePromiseResult(promise, rejectedTypeId, promiseResultTypeId, error, promiseReasonTypeId, handleInsertId);
                            promiseResults.push(promiseResult);
                        }
                    }
                    try {
                        yield deep.insert(promiseResults, { name: 'IMPORT_PROMISES_RESULTS' });
                        handleSelectorDebug("inserted promiseResults: ", JSON.stringify(promiseResults, null, 2));
                    }
                    catch (e) {
                        handleSelectorDebug('promiseResults insert error: ', (_l = e === null || e === void 0 ? void 0 : e.message) !== null && _l !== void 0 ? _l : e);
                    }
                }));
            }
        }
    });
}
exports.handleSelector = handleSelector;
function handleSchedule(handleScheduleLink, operation) {
    var _a, _b, _c, _d;
    return __awaiter(this, void 0, void 0, function* () {
        const handleScheduleDebug = (0, debug_1.default)('deeplinks:eh:links:handleSchedule');
        handleScheduleDebug('handleScheduleLink', handleScheduleLink);
        handleScheduleDebug('operation', operation);
        if (operation == 'INSERT') {
            const schedule = yield deep.select({
                type_id: yield deep.id('@deep-foundation/core', 'Schedule'),
                out: {
                    id: { _eq: handleScheduleLink.id },
                },
            }, {
                table: 'links',
                returning: 'id value',
            });
            handleScheduleDebug(schedule);
            const scheduleId = (_b = (_a = schedule === null || schedule === void 0 ? void 0 : schedule.data) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id;
            const scheduleValue = (_d = (_c = schedule === null || schedule === void 0 ? void 0 : schedule.data) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.value.value;
            handleScheduleDebug('scheduleId', scheduleId);
            handleScheduleDebug('scheduleValue', scheduleValue);
            yield exports.api.query({
                type: 'create_cron_trigger',
                args: {
                    name: `handle_schedule_${handleScheduleLink === null || handleScheduleLink === void 0 ? void 0 : handleScheduleLink.id}`,
                    webhook: `${DOCKER_DEEPLINKS_URL}/api/scheduler`,
                    schedule: scheduleValue,
                    include_in_metadata: true,
                    payload: {
                        scheduleId,
                        schedule: scheduleValue,
                        handleScheduleLinkId: handleScheduleLink === null || handleScheduleLink === void 0 ? void 0 : handleScheduleLink.id,
                    },
                    retry_conf: {
                        num_retries: 3,
                        timeout_seconds: 120,
                        tolerance_seconds: 21675,
                        retry_interval_seconds: 12
                    },
                    comment: `Event trigger for handle schedule link ${handleScheduleLink === null || handleScheduleLink === void 0 ? void 0 : handleScheduleLink.id} with cron schedule definition ${scheduleValue} of ${scheduleId} schedule.`,
                }
            });
            handleScheduleDebug('cron trigger created');
        }
        else if (operation == 'DELETE') {
            yield exports.api.query({
                type: 'delete_cron_trigger',
                args: {
                    name: `handle_schedule_${handleScheduleLink === null || handleScheduleLink === void 0 ? void 0 : handleScheduleLink.id}`,
                }
            });
            handleScheduleDebug('cron trigger deleted');
        }
    });
}
exports.handleSchedule = handleSchedule;
function handlePort(handlePortLink, operation) {
    var _a, _b, _c, _d, _e, _f;
    return __awaiter(this, void 0, void 0, function* () {
        const handlePortDebug = (0, debug_1.default)('deeplinks:eh:links:handlePort');
        handlePortDebug('handlePortLink', handlePortLink);
        handlePortDebug('operation', operation);
        const port = yield deep.select({
            id: { _eq: handlePortLink.from_id },
        }, {
            table: 'links',
            returning: 'id value',
        });
        handlePortDebug(port);
        const portId = (_b = (_a = port === null || port === void 0 ? void 0 : port.data) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id;
        const portValue = (_d = (_c = port === null || port === void 0 ? void 0 : port.data) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.value.value;
        handlePortDebug('portId', portId);
        handlePortDebug('portValue', portValue);
        if (operation == 'INSERT') {
            const isolationProvider = yield deep.select({
                type_id: yield deep.id('@deep-foundation/core', 'DockerIsolationProvider'),
                in: {
                    id: { _eq: handlePortLink.id },
                },
            }, {
                table: 'links',
                returning: 'id value',
            });
            handlePortDebug('INSERT', isolationProvider);
            const dockerImage = (_f = (_e = isolationProvider === null || isolationProvider === void 0 ? void 0 : isolationProvider.data) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.value.value;
            handlePortDebug('INSERT dockerImage', dockerImage);
            const containerName = `deep${yield containerController.getDelimiter()}handle_port_${portValue}`;
            handlePortDebug('INSERT containerName', containerName);
            const container = yield containerController.newContainer({ publish: true, forcePort: portValue, forceName: containerName, handler: dockerImage, code: null, jwt: null, data: {} });
            handlePortDebug('INSERT newContainer result', container);
            if (container.error)
                return handlePortDebug('portResult.error', container.error);
            handlePortDebug(`INSERT port handler container ${JSON.stringify(container)} created`);
        }
        else if (operation == 'DELETE') {
            const containerName = `deep${yield containerController.getDelimiter()}handle_port_${portValue}`;
            handlePortDebug('DELETE containerName', containerName);
            const container = yield containerController.findContainer(containerName);
            handlePortDebug('DELETE container', container);
            yield containerController.dropContainer(container);
            handlePortDebug('DELETE port handler container deleted');
        }
    });
}
exports.handlePort = handlePort;
exports.default = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d, _e, _f, _g;
    try {
        let handlePortId;
        try {
            handlePortId = yield deep.id('@deep-foundation/core', 'HandlePort');
        }
        catch (_h) {
            return res.status(500).json({ error: '@deep-foundation/core package is not ready to support links handlers.' });
        }
        const event = (_a = req === null || req === void 0 ? void 0 : req.body) === null || _a === void 0 ? void 0 : _a.event;
        const operation = event === null || event === void 0 ? void 0 : event.op;
        if (operation === 'INSERT' || operation === 'UPDATE' || operation === 'DELETE') {
            const oldRow = (_b = event === null || event === void 0 ? void 0 : event.data) === null || _b === void 0 ? void 0 : _b.old;
            if (oldRow) {
                const queryResult = yield deep.select({
                    id: { _eq: oldRow.id },
                }, {
                    returning: `value`,
                });
                oldRow.value = (_d = (_c = queryResult.data) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.value;
            }
            const newRow = (_e = event === null || event === void 0 ? void 0 : event.data) === null || _e === void 0 ? void 0 : _e.new;
            if (newRow) {
                const queryResult = yield deep.select({
                    id: { _eq: newRow.id },
                }, {
                    returning: `value`,
                });
                newRow.value = (_g = (_f = queryResult.data) === null || _f === void 0 ? void 0 : _f[0]) === null || _g === void 0 ? void 0 : _g.value;
            }
            const current = operation === 'DELETE' ? oldRow : newRow;
            log(`Processing ${current.id} link.`);
            log('operation', operation);
            log('current', current);
            try {
                if (operation === 'INSERT') {
                    yield handleOperation('Insert', oldRow, newRow);
                    yield handleSelector(oldRow, newRow);
                }
                else if (operation === 'UPDATE') {
                }
                else if (operation === 'DELETE') {
                    yield handleOperation('Delete', oldRow, newRow);
                }
                const typeId = current.type_id;
                const handleScheduleId = yield deep.id('@deep-foundation/core', 'HandleSchedule');
                if (typeId === handleScheduleId && (operation === 'INSERT' || operation === 'DELETE')) {
                    yield handleSchedule(current, operation);
                }
                if (typeId === handlePortId && (operation === 'INSERT' || operation === 'DELETE')) {
                    yield handlePort(current, operation);
                }
                log(`Link ${current.id} is proccessed.`);
                if (operation === 'INSERT' && !global_ids_1.DENIED_IDS.includes(current.type_id) && global_ids_1.ALLOWED_IDS.includes(current.type_id)) {
                    log('resolve', current.id);
                    yield (0, promise_1.resolve)({
                        id: current.id, client,
                        Then: yield deep.id('@deep-foundation/core', 'Then'),
                        Promise: yield deep.id('@deep-foundation/core', 'Promise'),
                        Resolved: yield deep.id('@deep-foundation/core', 'Resolved'),
                        Rejected: yield deep.id('@deep-foundation/core', 'Rejected'),
                        Results: false,
                    });
                }
                return res.status(200).json({});
            }
            catch (e) {
                error('error', e);
                if (operation === 'INSERT' && !global_ids_1.DENIED_IDS.includes(current.type_id) && global_ids_1.ALLOWED_IDS.includes(current.type_id)) {
                    log('reject', current.id);
                    yield (0, promise_1.reject)({
                        id: current.id, client,
                        Then: yield deep.id('@deep-foundation/core', 'Then'),
                        Promise: yield deep.id('@deep-foundation/core', 'Promise'),
                        Resolved: yield deep.id('@deep-foundation/core', 'Resolved'),
                        Rejected: yield deep.id('@deep-foundation/core', 'Rejected'),
                        Results: false,
                    });
                }
            }
            return res.status(500).json({ error: 'notexplained' });
        }
        return res.status(500).json({ error: 'operation can be only INSERT or UPDATE' });
    }
    catch (e) {
        return res.status(500).json({ error: e.toString() });
    }
});
//# sourceMappingURL=links.js.map