"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteSelector = exports.insertSelectorItem = exports.insertSelector = exports.getPromiseResults = exports.ensureLinkIsCreated = exports.deleteScheduleHandler = exports.deleteHandler = exports.deletePromiseResult = exports.deleteIds = exports.deleteId = void 0;
const client_1 = require("@deep-foundation/hasura/client");
const client_2 = require("../imports/client");
const chai_1 = require("chai");
const graphql_tag_1 = __importDefault(require("graphql-tag"));
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('deeplinks:tests:handlers');
const log = debug.extend('log');
const error = debug.extend('error');
const namespaces = debug_1.default.disable();
debug_1.default.enable(`${namespaces ? `${namespaces},` : ``}${error.namespace}`);
const wait_on_1 = __importDefault(require("wait-on"));
const get_port_1 = __importDefault(require("get-port"));
const apolloClient = (0, client_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_2.DeepClient({ apolloClient });
const DELAY = +process.env.DELAY || 0;
const delay = time => new Promise(res => setTimeout(res, time));
const insertHandler = (handleOperationTypeId, typeId, code) => __awaiter(void 0, void 0, void 0, function* () {
    const handlerTypeId = yield deep.id('@deep-foundation/core', 'Handler');
    const syncTextFileTypeId = yield deep.id('@deep-foundation/core', 'SyncTextFile');
    const isolationProviderThatSupportsJSExecutionProviderId = yield deep.id('@deep-foundation/core', 'dockerSupportsJs');
    const handlerJSFile = (yield deep.insert({
        type_id: syncTextFileTypeId,
    }, { name: 'IMPORT_HANDLER_JS_FILE' })).data[0];
    const handlerJSFileValue = (yield deep.insert({ link_id: handlerJSFile === null || handlerJSFile === void 0 ? void 0 : handlerJSFile.id, value: code }, { table: 'strings' })).data[0];
    const handler = (yield deep.insert({
        from_id: isolationProviderThatSupportsJSExecutionProviderId,
        type_id: handlerTypeId,
        to_id: handlerJSFile === null || handlerJSFile === void 0 ? void 0 : handlerJSFile.id,
    }, { name: 'IMPORT_HANDLER' })).data[0];
    const handleOperation = (yield deep.insert({
        from_id: typeId,
        type_id: handleOperationTypeId,
        to_id: handler === null || handler === void 0 ? void 0 : handler.id,
    }, { name: 'IMPORT_INSERT_HANDLER' })).data[0];
    return {
        handlerId: handler === null || handler === void 0 ? void 0 : handler.id,
        handleOperationId: handleOperation === null || handleOperation === void 0 ? void 0 : handleOperation.id,
        handlerJSFileId: handlerJSFile === null || handlerJSFile === void 0 ? void 0 : handlerJSFile.id,
        handlerJSFileValueId: handlerJSFileValue === null || handlerJSFileValue === void 0 ? void 0 : handlerJSFileValue.id,
    };
});
const insertOperationHandlerForSchedule = (schedule, code) => __awaiter(void 0, void 0, void 0, function* () {
    const handlerTypeId = yield deep.id('@deep-foundation/core', 'Handler');
    const scheduleTypeId = yield deep.id('@deep-foundation/core', 'Schedule');
    const handleScheduleTypeId = yield deep.id('@deep-foundation/core', 'HandleSchedule');
    const syncTextFileTypeId = yield deep.id('@deep-foundation/core', 'SyncTextFile');
    const isolationProviderThatSupportsJSExecutionProviderId = yield deep.id('@deep-foundation/core', 'dockerSupportsJs');
    const handlerJSFile = (yield deep.insert({
        type_id: syncTextFileTypeId,
    }, { name: 'IMPORT_HANDLER_JS_FILE' })).data[0];
    const handlerJSFileValue = (yield deep.insert({ link_id: handlerJSFile === null || handlerJSFile === void 0 ? void 0 : handlerJSFile.id, value: code }, { table: 'strings' })).data[0];
    const handler = (yield deep.insert({
        from_id: isolationProviderThatSupportsJSExecutionProviderId,
        type_id: handlerTypeId,
        to_id: handlerJSFile === null || handlerJSFile === void 0 ? void 0 : handlerJSFile.id,
    }, { name: 'IMPORT_HANDLER' })).data[0];
    const scheduleNode = (yield deep.insert({
        type_id: scheduleTypeId,
    }, { name: 'IMPORT_SCHEDULE' })).data[0];
    const scheduleValue = (yield deep.insert({ link_id: scheduleNode === null || scheduleNode === void 0 ? void 0 : scheduleNode.id, value: schedule }, { table: 'strings' })).data[0];
    const handleOperation = (yield deep.insert({
        from_id: scheduleNode === null || scheduleNode === void 0 ? void 0 : scheduleNode.id,
        type_id: handleScheduleTypeId,
        to_id: handler === null || handler === void 0 ? void 0 : handler.id,
    }, { name: 'IMPORT_INSERT_HANDLER' })).data[0];
    return {
        handlerId: handler === null || handler === void 0 ? void 0 : handler.id,
        handleOperationId: handleOperation === null || handleOperation === void 0 ? void 0 : handleOperation.id,
        handlerJSFileId: handlerJSFile === null || handlerJSFile === void 0 ? void 0 : handlerJSFile.id,
        handlerJSFileValueId: handlerJSFileValue === null || handlerJSFileValue === void 0 ? void 0 : handlerJSFileValue.id,
        scheduleId: scheduleNode === null || scheduleNode === void 0 ? void 0 : scheduleNode.id,
        scheduleValueId: scheduleValue === null || scheduleValue === void 0 ? void 0 : scheduleValue.id,
    };
});
function deleteId(id, options = { table: 'links' }) {
    return __awaiter(this, void 0, void 0, function* () {
        yield deleteIds([id], options);
    });
}
exports.deleteId = deleteId;
function deleteIds(ids, options = { table: 'links' }) {
    return __awaiter(this, void 0, void 0, function* () {
        const idsFiltered = ids === null || ids === void 0 ? void 0 : ids.filter(linkId => typeof linkId === 'number');
        if ((idsFiltered === null || idsFiltered === void 0 ? void 0 : idsFiltered.length) > 0) {
            try {
                return yield deep.delete(idsFiltered, options);
            }
            catch (e) {
                error(`Error deleting ids: ${idsFiltered.join(', ')}`, JSON.stringify(e, null, 2));
            }
        }
        else {
            return { data: [] };
        }
    });
}
exports.deleteIds = deleteIds;
function deletePromiseResult(promiseResult, linkId) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q;
    return __awaiter(this, void 0, void 0, function* () {
        const resultLinkId = (_b = (_a = promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.in) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id;
        const thenLinkId = (_g = (_f = (_e = (_d = (_c = promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.in) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.from) === null || _e === void 0 ? void 0 : _e.in) === null || _f === void 0 ? void 0 : _f[0]) === null || _g === void 0 ? void 0 : _g.id;
        const valueId = (_h = promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.object) === null || _h === void 0 ? void 0 : _h.id;
        const promiseResultId = promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.id;
        const promiseId = (_l = (_k = (_j = promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.in) === null || _j === void 0 ? void 0 : _j[0]) === null || _k === void 0 ? void 0 : _k.from) === null || _l === void 0 ? void 0 : _l.id;
        const promiseReasonId = (_q = (_p = (_o = (_m = promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.in) === null || _m === void 0 ? void 0 : _m[0]) === null || _o === void 0 ? void 0 : _o.out) === null || _p === void 0 ? void 0 : _p[0]) === null || _q === void 0 ? void 0 : _q.id;
        yield deleteId(promiseReasonId);
        yield deleteIds([resultLinkId, thenLinkId]);
        yield deleteIds([promiseResultId, promiseId, linkId]);
        yield deleteId(valueId, { table: 'objects' });
    });
}
exports.deletePromiseResult = deletePromiseResult;
const deleteHandler = (handler) => __awaiter(void 0, void 0, void 0, function* () {
    yield deleteIds([handler.handlerJSFileId, handler.handlerId, handler.handleOperationId]);
    yield deleteId(handler.handlerJSFileValueId, { table: 'strings' });
});
exports.deleteHandler = deleteHandler;
const deleteScheduleHandler = (handler) => __awaiter(void 0, void 0, void 0, function* () {
    yield (0, exports.deleteHandler)(handler);
    yield deleteId(handler.scheduleId);
    yield deleteId(handler.scheduleValueId, { table: 'strings' });
});
exports.deleteScheduleHandler = deleteScheduleHandler;
function ensureLinkIsCreated(typeId) {
    return __awaiter(this, void 0, void 0, function* () {
        const freeId = randomInteger(5000000, 9999999999);
        const insertedLink = (yield deep.insert({
            id: freeId,
            from_id: freeId,
            type_id: typeId,
            to_id: freeId
        }, { name: 'IMPORT_LINK' })).data[0];
        chai_1.assert.equal(freeId, insertedLink.id);
        return freeId;
    });
}
exports.ensureLinkIsCreated = ensureLinkIsCreated;
function getPromiseResults(deep, resultTypeId, linkId) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const promiseTypeId = yield deep.id('@deep-foundation/core', 'Promise');
        const thenTypeId = yield deep.id('@deep-foundation/core', 'Then');
        const promiseReasonTypeId = yield deep.id('@deep-foundation/core', 'PromiseReason');
        const client = deep.apolloClient;
        const queryString = `{
    links(where: { 
      in: {
        type_id: { _eq: ${resultTypeId} }, # Resolved/Rejected
        from: { 
          type_id: { _eq: ${promiseTypeId} }, # Promise
          in: { 
            type_id: { _eq: ${thenTypeId} } # Then
            from_id: { _eq: ${linkId} } # linkId
          }
        }
      },
    }) {
      id
      object {
        id
        value
      }
      in(where: { type_id: { _eq: ${resultTypeId} } }) {
        id
        from {
          id
          in(where: { type_id: { _eq: ${thenTypeId} } }) {
            id
          }
        }
        out(where: { type_id: { _eq: ${promiseReasonTypeId} } }) {
          id
        }
      }
    }
  }`;
        return (_b = (_a = (yield client.query({
            query: (0, graphql_tag_1.default) `${queryString}`,
        }))) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.links;
    });
}
exports.getPromiseResults = getPromiseResults;
jest.setTimeout(120000);
function randomInteger(min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
}
describe('sync function handle by type with resolve', () => {
    it(`handle insert`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = randomInteger(5000000, 9999999999);
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
        const handler = yield insertHandler(handleInsertTypeId, typeId, `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        const linkId = yield ensureLinkIsCreated(typeId);
        yield deep.await(linkId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults = yield getPromiseResults(deep, resolvedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, exports.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
    it(`handle update when value is inserted`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = randomInteger(5000000, 9999999999);
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleUpdateTypeId = yield deep.id('@deep-foundation/core', 'HandleUpdate');
        const handler = yield insertHandler(handleUpdateTypeId, typeId, `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        const linkId = yield ensureLinkIsCreated(typeId);
        yield deep.insert({ link_id: linkId, value: numberToReturn }, { table: 'numbers' });
        yield deep.await(linkId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        let promiseResults = yield getPromiseResults(deep, resolvedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, exports.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
    it(`handle update when value is updated`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = randomInteger(5000000, 9999999999);
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleUpdateTypeId = yield deep.id('@deep-foundation/core', 'HandleUpdate');
        const handler = yield insertHandler(handleUpdateTypeId, typeId, `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        const linkId = yield ensureLinkIsCreated(typeId);
        yield deep.insert({ link_id: linkId, value: numberToReturn }, { table: 'numbers' });
        yield deep.await(linkId);
        yield deep.update({ link_id: linkId }, { value: numberToReturn + 1 }, { table: 'numbers' });
        yield deep.await(linkId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults = yield getPromiseResults(deep, resolvedTypeId, linkId);
        const matchedPromiseResults = promiseResults.filter(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        for (const promiseResult of matchedPromiseResults) {
            yield deletePromiseResult(promiseResult);
        }
        yield deleteId(linkId);
        yield (0, exports.deleteHandler)(handler);
        chai_1.assert.isTrue(!!matchedPromiseResults);
        chai_1.assert.equal(matchedPromiseResults.length, 2);
    }));
    it(`handle update when value is deleted`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = randomInteger(5000000, 9999999999);
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleUpdateTypeId = yield deep.id('@deep-foundation/core', 'HandleUpdate');
        const handler = yield insertHandler(handleUpdateTypeId, typeId, `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        const linkId = yield ensureLinkIsCreated(typeId);
        yield deep.insert({ link_id: linkId, value: numberToReturn }, { table: 'numbers' });
        yield deep.await(linkId);
        yield deep.delete({ link_id: { _eq: linkId } }, { table: 'numbers' });
        yield deep.await(linkId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults = yield getPromiseResults(deep, resolvedTypeId, linkId);
        const matchedPromiseResults = promiseResults.filter(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        for (const promiseResult of matchedPromiseResults) {
            yield deletePromiseResult(promiseResult);
        }
        yield deleteId(linkId);
        yield (0, exports.deleteHandler)(handler);
        chai_1.assert.isTrue(!!matchedPromiseResults);
        chai_1.assert.equal(matchedPromiseResults.length, 2);
    }));
    it(`handle delete`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = randomInteger(5000000, 9999999999);
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleDeleteTypeId = yield deep.id('@deep-foundation/core', 'HandleDelete');
        const handler = yield insertHandler(handleDeleteTypeId, typeId, `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        const linkId = yield ensureLinkIsCreated(typeId);
        yield deleteId(linkId);
        yield deep.await(linkId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults = yield getPromiseResults(deep, resolvedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, exports.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
});
describe('sync function handle by type with reject', () => {
    it(`handle insert`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToThrow = randomInteger(5000000, 9999999999);
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
        const handler = yield insertHandler(handleInsertTypeId, typeId, `(arg) => { throw ${numberToThrow}; return { "error": "return is not possible" }; }`);
        const linkId = yield ensureLinkIsCreated(typeId);
        yield deep.await(linkId);
        const rejectedTypeId = yield deep.id('@deep-foundation/core', 'Rejected');
        const promiseResults = yield getPromiseResults(deep, rejectedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a; return ((_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === numberToThrow; });
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, exports.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
    it(`handle delete`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToThrow = randomInteger(5000000, 9999999999);
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleDeleteTypeId = yield deep.id('@deep-foundation/core', 'HandleDelete');
        const handler = yield insertHandler(handleDeleteTypeId, typeId, `(arg) => { throw ${numberToThrow}; return { "error": "return is not possible" }; }`);
        const linkId = yield ensureLinkIsCreated(typeId);
        yield deleteId(linkId);
        yield deep.await(linkId);
        const rejectedTypeId = yield deep.id('@deep-foundation/core', 'Rejected');
        const promiseResults = yield getPromiseResults(deep, rejectedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a; return ((_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === numberToThrow; });
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, exports.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
});
describe('async function handle by type with reject', () => {
    it(`handle insert`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToThrow = randomInteger(5000000, 9999999999);
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
        const handler = yield insertHandler(handleInsertTypeId, typeId, `async (arg) => { throw ${numberToThrow}; return { "error": "return is not possible" }; }`);
        const linkId = yield ensureLinkIsCreated(typeId);
        yield deep.await(linkId);
        const rejectedTypeId = yield deep.id('@deep-foundation/core', 'Rejected');
        const promiseResults = yield getPromiseResults(deep, rejectedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a; return ((_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === numberToThrow; });
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, exports.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
    it(`handle delete`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToThrow = randomInteger(5000000, 9999999999);
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleDeleteTypeId = yield deep.id('@deep-foundation/core', 'HandleDelete');
        const handler = yield insertHandler(handleDeleteTypeId, typeId, `async (arg) => { throw ${numberToThrow}; return { "error": "return is not possible" }; }`);
        const linkId = yield ensureLinkIsCreated(typeId);
        yield deleteId(linkId);
        yield deep.await(linkId);
        const rejectedTypeId = yield deep.id('@deep-foundation/core', 'Rejected');
        const promiseResults = yield getPromiseResults(deep, rejectedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a; return ((_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === numberToThrow; });
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, exports.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
});
describe('sync function handle by schedule with resolve', () => {
    it(`handle schedule`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = randomInteger(5000000, 9999999999);
        const handler = yield insertOperationHandlerForSchedule('* * * * *', `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        yield deep.await(handler.scheduleId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults = yield getPromiseResults(deep, resolvedTypeId, handler.scheduleId);
        const promiseResult = promiseResults.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        yield deletePromiseResult(promiseResult, handler.scheduleId);
        yield (0, exports.deleteScheduleHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
});
describe('async function handle by type with resolve using deep client', () => {
    it(`handle insert`, () => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b;
        const numberToReturn = randomInteger(5000000, 9999999999);
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
        const queryTypeId = yield deep.id('@deep-foundation/core', 'Query');
        const handler = yield insertHandler(handleInsertTypeId, typeId, `async (arg) => {
       const deep = arg.deep;
       const queryTypeId = await deep.id('@deep-foundation/core', 'Query');
       const queryId = (await deep.insert({ type_id: queryTypeId }))?.data?.[0]?.id;
      //  const queryId = (await deep.insert({ type_id: ${queryTypeId} }))?.data?.[0]?.id;
       return { queryId, result: ${numberToReturn}}
    }`);
        const linkId = yield ensureLinkIsCreated(typeId);
        yield deep.await(linkId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults = yield getPromiseResults(deep, resolvedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        const queryId = (_b = (_a = promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.queryId;
        const query = (yield deep.select({ id: { _eq: queryId } })).data[0];
        yield deleteId(queryId);
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, exports.deleteHandler)(handler);
        chai_1.assert.equal(query.id, queryId);
        chai_1.assert.isTrue(!!promiseResult);
    }));
});
describe('handle port', () => {
    it(`handle port`, () => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b, _c, _d, _e, _f;
        const port = yield (0, get_port_1.default)();
        const portTypeId = yield deep.id('@deep-foundation/core', 'Port');
        const portId = (_c = (_b = (_a = (yield deep.insert({
            type_id: portTypeId,
            number: { data: { value: port } }
        }))) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.id;
        const jsDockerIsolationProviderId = yield deep.id('@deep-foundation/core', 'JSDockerIsolationProvider');
        const handlePortTypeId = yield deep.id('@deep-foundation/core', 'HandlePort');
        const hanlePortLinkId = (_f = (_e = (_d = (yield deep.insert({ from_id: portId, type_id: handlePortTypeId, to_id: jsDockerIsolationProviderId }))) === null || _d === void 0 ? void 0 : _d.data) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.id;
        yield deep.await(hanlePortLinkId);
        log("waiting for container to be created");
        yield (0, wait_on_1.default)({ resources: [`http://localhost:${port}/healthz`] });
        log("container is up");
        yield deleteId(hanlePortLinkId);
        log("waiting for container to be removed");
        yield (0, wait_on_1.default)({
            resources: [
                `http://localhost:${port}/healthz`
            ],
            reverse: true,
        });
        log("container is down");
    }));
});
function insertSelector() {
    return __awaiter(this, void 0, void 0, function* () {
        const typeTypeId = yield deep.id('@deep-foundation/core', 'Type');
        const { data: [{ id: ty0 }] } = yield deep.insert({
            type_id: typeTypeId,
        });
        const { data: [{ id: ty1 }] } = yield deep.insert({
            type_id: typeTypeId,
            from_id: ty0,
            to_id: ty0,
        });
        const treeIncludeDownTypeId = yield deep.id('@deep-foundation/core', 'TreeIncludeDown');
        const { data: [{ id: tr1, out: treeIncludes }] } = yield deep.insert({
            type_id: yield deep.id('@deep-foundation/core', 'Tree'),
            out: { data: [
                    {
                        type_id: treeIncludeDownTypeId,
                        to_id: ty1,
                    },
                    {
                        type_id: treeIncludeDownTypeId,
                        to_id: ty0,
                    },
                ] }
        }, {
            returning: `
      id
      out(where: { to_id: { _in: [${ty0}, ${ty1}] }, type_id: { _eq: ${treeIncludeDownTypeId} } }) {
        id
      }
    `,
        });
        const { data: [{ id: id0 }] } = yield deep.insert({
            type_id: ty0,
        });
        const selectorIncludeTypeId = yield deep.id('@deep-foundation/core', 'SelectorInclude');
        const selectorTreeTypeId = yield deep.id('@deep-foundation/core', 'SelectorTree');
        const { data: [{ id: s1, out: selectorData }] } = yield deep.insert({
            type_id: yield deep.id('@deep-foundation/core', 'Selector'),
            out: { data: [
                    {
                        type_id: selectorIncludeTypeId,
                        to_id: id0,
                        out: { data: {
                                type_id: selectorTreeTypeId,
                                to_id: tr1
                            } },
                    },
                ] }
        }, {
            returning: `
      id
      out(where: { to_id: { _eq: ${id0} }, type_id: { _eq: ${selectorIncludeTypeId} } }) {
        id
        out(where: { to_id: { _eq: ${tr1} }, type_id: { _eq: ${selectorTreeTypeId} } }) {
          id
        }
      }
    `,
        });
        return {
            nodeTypeId: ty0,
            linkTypeId: ty1,
            treeId: tr1,
            treeIncludesIds: treeIncludes.map(({ id }) => id),
            selectorId: s1,
            selectorIncludeId: selectorData[0].id,
            selectorTreeId: selectorData[0].out[0].id,
            rootId: id0,
        };
    });
}
exports.insertSelector = insertSelector;
;
function insertSelectorItem({ selectorId, nodeTypeId, linkTypeId, treeId, rootId }) {
    return __awaiter(this, void 0, void 0, function* () {
        const { data: [{ id: linkId1, to: { id: nodeId1 } }] } = yield deep.insert({
            type_id: linkTypeId,
            from_id: rootId,
            to: { data: {
                    type_id: nodeTypeId,
                } }
        }, { returning: 'id to { id }' });
        const { data: [{ id: linkId2, to: { id: nodeId2 } }] } = yield deep.insert({
            from_id: nodeId1,
            type_id: linkTypeId,
            to: { data: {
                    type_id: nodeTypeId,
                } }
        }, { returning: 'id to { id }' });
        return [
            { linkId: linkId1, nodeId: nodeId1 },
            { linkId: linkId2, nodeId: nodeId2 },
        ];
    });
}
exports.insertSelectorItem = insertSelectorItem;
;
const deleteSelector = (selector) => __awaiter(void 0, void 0, void 0, function* () {
    yield deleteId(selector.linkTypeId);
    yield deleteId(selector.nodeTypeId);
    yield deleteId(selector.treeId);
    yield deleteIds(selector.treeIncludesIds);
    yield deleteId(selector.selectorId);
    yield deleteId(selector.selectorIncludeId);
    yield deleteId(selector.selectorTreeId);
    yield deleteId(selector.rootId);
});
exports.deleteSelector = deleteSelector;
describe.skip('handle by selector', () => {
    it(`handle insert`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = randomInteger(5000000, 9999999999);
        const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
        const selector = yield insertSelector();
        const { nodeTypeId, linkTypeId, treeId, treeIncludesIds, selectorId, selectorIncludeId, selectorTreeId, rootId } = selector;
        const handler = yield insertHandler(handleInsertTypeId, selectorId, `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        const selectorItems = yield insertSelectorItem({ selectorId, nodeTypeId, linkTypeId, treeId, rootId });
        log(`awaiting ${selectorItems[1].linkId} link.`);
        yield deep.await(selectorItems[1].linkId);
        log(`awaiting ${selectorItems[0].linkId} link.`);
        yield deep.await(selectorItems[0].linkId);
        const resolvedTypeId1 = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults1 = yield getPromiseResults(deep, resolvedTypeId1, selectorItems[1].linkId);
        const promiseResult1 = promiseResults1.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        console.log('promiseResult1', JSON.stringify(promiseResult1, null, 2));
        const resolvedTypeId2 = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults2 = yield getPromiseResults(deep, resolvedTypeId2, selectorItems[0].linkId);
        const promiseResult2 = promiseResults2.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        console.log('promiseResult2', JSON.stringify(promiseResult2, null, 2));
        for (const selectorItem of selectorItems) {
            yield deleteId(selectorItem.linkId);
            yield deleteId(selectorItem.nodeId);
        }
        yield (0, exports.deleteSelector)(selector);
        yield deletePromiseResult(promiseResult1);
        yield deletePromiseResult(promiseResult2);
        yield (0, exports.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult1);
        chai_1.assert.isTrue(!!promiseResult2);
    }));
});
describe.skip('debug', () => {
    it(`log`, () => __awaiter(void 0, void 0, void 0, function* () {
        log('this message should be log');
    }));
    it(`error`, () => __awaiter(void 0, void 0, void 0, function* () {
        error('this message should be error');
    }));
});
//# sourceMappingURL=handlers.js.map